$CONFIG_SH='config.sh'
$Id='$Id'
$alignbytes='8'
$aphostname=''
$archlib=''
$archlibexp=''
$archname=''
$asctime_r_proto='0'
$bin='/system/ported/command_library'
$binexp='/system/ported/command_library'
$byteorder='4321'
$castflags='0'
$cf_by='Paul.Green@stratus.com'
$cf_time='2004-01-13 12:00 UCT'
$cpp_stuff='42'
$cpplast='-'
$cppminus='-'
$cpprun='cc -E -'
$cppstdin='cc -E'
$crypt_r_proto='0'
$ctermid_r_proto='0'
$ctime_r_proto='0'
$d_Gconvert='sprintf((b),"%.*g",(n),(x))'
$d_PRIeldbl='define'
$d_PRIfldbl='define'
$d_PRIgldbl='define'
$d_SCNfldbl='define'
$d__fwalk='undef'
$d_access='undef'
$d_accessx='undef'
$d_aintl='undef'
$d_alarm='define'
$d_archlib='undef'
$d_asctime_r='undef'
$d_atolf='undef'
$d_atoll='undef'
$d_attribut='undef'
$d_bcmp='undef'
$d_bcopy='undef'
$d_bsdgetpgrp='undef'
$d_bsdsetpgrp='undef'
$d_bzero='undef'
$d_casti32='undef'
$d_castneg='define'
$d_charvspr='undef'
$d_chown='undef'
$d_chroot='undef'
$d_chsize='undef'
$d_class='undef'
$d_cmsghdr_s='undef'
$d_const='define'
$d_copysignl='undef'
$d_crypt='undef'
$d_crypt_r='undef'
$d_csh='undef'
$d_ctermid_r='undef'
$d_ctime_r='undef'
$d_cuserid='undef'
$d_dbl_dig='define'
$d_dbminitproto='undef'
$d_difftime='define'
$d_dirfd='undef'
$d_dirnamlen='undef'
$d_dlerror='undef'
$d_dlsymun='undef'
$d_dosuid='undef'
$d_drand48_r='undef'
$d_drand48proto='undef'
$d_dup2='undef'
$d_eaccess='undef'
$d_endgrent='undef'
$d_endgrent_r='undef'
$d_endhent='define'
$d_endhostent_r='undef'
$d_endnent='define'
$d_endnetent_r='undef'
$d_endpent='define'
$d_endprotoent_r='undef'
$d_endpwent='undef'
$d_endpwent_r='undef'
$d_endsent='define'
$d_endservent_r='undef'
$d_eofnblk='define'
$d_faststdio='define'
$d_fchdir='undef'
$d_fchmod='define'
$d_fchown='undef'
$d_fcntl='define'
$d_fcntl_can_lock='define'
$d_fd_set='define'
$d_fgetpos='define'
$d_finite='undef'
$d_finitel='undef'
$d_flexfnam='define'
$d_flock='undef'
$d_flockproto='undef'
$d_fork='undef'
$d_fp_class='undef'
$d_fpathconf='define'
$d_fpclass='undef'
$d_fpclassify='undef'
$d_fpclassl='undef'
$d_fpos64_t='undef'
$d_frexpl='undef'
$d_fs_data_s='undef'
$d_fseeko='undef'
$d_fsetpos='define'
$d_fstatfs='undef'
$d_fstatvfs='undef'
$d_fsync='undef'
$d_ftello='undef'
$d_getcwd='define'
$d_getespwnam='undef'
$d_getfsstat='undef'
$d_getgrent='undef'
$d_getgrent_r='undef'
$d_getgrgid_r='undef'
$d_getgrnam_r='undef'
$d_getgrps='undef'
$d_gethbyaddr='define'
$d_gethbyname='define'
$d_gethent='define'
$d_gethname='define'
$d_gethostbyaddr_r='undef'
$d_gethostbyname_r='undef'
$d_gethostent_r='undef'
$d_gethostprotos='define'
$d_getitimer='undef'
$d_getlogin='define'
$d_getlogin_r='undef'
$d_getmnt='undef'
$d_getmntent='undef'
$d_getnbyaddr='define'
$d_getnbyname='define'
$d_getnent='define'
$d_getnetbyaddr_r='undef'
$d_getnetbyname_r='undef'
$d_getnetent_r='undef'
$d_getnetprotos='define'
$d_getpagsz='undef'
$d_getpbyname='define'
$d_getpbynumber='define'
$d_getpent='define'
$d_getpgid='undef'
$d_getpgrp2='undef'
$d_getpgrp='define'
$d_getppid='define'
$d_getprior='undef'
$d_getprotobyname_r='undef'
$d_getprotobynumber_r='undef'
$d_getprotoent_r='undef'
$d_getprotoprotos='define'
$d_getprpwnam='undef'
$d_getpwent='undef'
$d_getpwent_r='undef'
$d_getpwnam_r='undef'
$d_getpwuid_r='undef'
$d_getsbyname='define'
$d_getsbyport='define'
$d_getsent='define'
$d_getservbyname_r='undef'
$d_getservbyport_r='undef'
$d_getservent_r='undef'
$d_getservprotos='define'
$d_getspnam='undef'
$d_getspnam_r='undef'
$d_gettimeod='undef'
$d_gmtime_r='undef'
$d_gnulibc='undef'
$d_grpasswd='undef'
$d_hasmntopt='undef'
$d_htonl='define'
$d_ilogbl='undef'
$d_index='undef'
$d_inetaton='undef'
$d_int64_t='undef'
$d_isascii='define'
$d_isfinite='undef'
$d_isinf='undef'
$d_isnan='undef'
$d_isnanl='undef'
$d_killpg='undef'
$d_lchown='undef'
$d_ldbl_dig='define'
$d_link='undef'
$d_localtime_r='undef'
$d_locconv='define'
$d_lockf='define'
$d_longdbl='define'
$d_longlong='undef'
$d_lseekproto='define'
$d_lstat='define'
$d_madvise='undef'
$d_mblen='define'
$d_mbstowcs='define'
$d_mbtowc='define'
$d_memchr='define'
$d_memcmp='define'
$d_memcpy='define'
$d_memmove='define'
$d_memset='define'
$d_mkdir='define'
$d_mkdtemp='undef'
$d_mkfifo='define'
$d_mkstemp='undef'
$d_mkstemps='undef'
$d_mktime='define'
$d_mmap='undef'
$d_modfl='undef'
$d_modflproto='undef'
$d_modfl_pow32_bug='undef'
$d_mprotect='undef'
$d_msg='undef'
$d_msg_ctrunc='undef'
$d_msg_dontroute='undef'
$d_msg_oob='undef'
$d_msg_peek='undef'
$d_msg_proxy='undef'
$d_msghdr_s='undef'
$d_msync='undef'
$d_munmap='undef'
$d_mymalloc='undef'
$d_nice='undef'
$d_nl_langinfo='undef'
$d_nv_preserves_uv='define'
$nv_preserves_uv_bits='32'
$d_off64_t='undef'
$d_old_pthread_create_joinable='undef'
$d_oldpthreads='undef'
$d_open3='define'
$d_pathconf='define'
$d_pause='define'
$d_perl_otherlibdirs='undef'
$d_phostname='undef'
$d_pipe='define'
$d_poll='define'
$d_procselfexe='undef'
$d_pthread_atfork='undef'
$d_pthread_attr_setscope='undef'
$d_pthread_yield='undef'
$d_pwage='undef'
$d_pwchange='undef'
$d_pwclass='undef'
$d_pwcomment='undef'
$d_pwexpire='undef'
$d_pwgecos='undef'
$d_pwpasswd='undef'
$d_pwquota='undef'
$d_quad='undef'
$d_random_r='undef'
$d_readdir64_r='undef'
$d_readdir='define'
$d_readdir_r='undef'
$d_readlink='define'
$d_readv='undef'
$d_recvmsg='undef'
$d_rename='define'
$d_rewinddir='define'
$d_rmdir='define'
$d_safebcpy='undef'
$d_safemcpy='undef'
$d_sanemcmp='define'
$d_sbrkproto='undef'
$d_scalbnl='undef'
$d_sched_yield='undef'
$d_scm_rights='undef'
$d_seekdir='undef'
$d_select='define'
$d_sem='undef'
$d_semctl_semid_ds='undef'
$d_semctl_semun='undef'
$d_sendmsg='undef'
$d_setegid='undef'
$d_seteuid='undef'
$d_setgrent='undef'
$d_setgrent_r='undef'
$d_setgrps='undef'
$d_sethent='define'
$d_sethostent_r='undef'
$d_setitimer='undef'
$d_setlinebuf='undef'
$d_setlocale='define'
$d_setlocale_r='undef'
$d_setnent='define'
$d_setnetent_r='undef'
$d_setpent='define'
$d_setpgid='undef'
$d_setpgrp2='undef'
$d_setpgrp='undef'
$d_setprior='undef'
$d_setproctitle='undef'
$d_setprotoent_r='undef'
$d_setpwent='undef'
$d_setpwent_r='undef'
$d_setregid='undef'
$d_setresgid='undef'
$d_setresuid='undef'
$d_setreuid='undef'
$d_setrgid='undef'
$d_setruid='undef'
$d_setsent='define'
$d_setservent_r='undef'
$d_setsid='undef'
$d_setvbuf='define'
$d_sfio='undef'
$d_shm='undef'
$d_shmatprototype='define'
$d_sigaction='undef'
$d_sigprocmask='undef'
$d_sigsetjmp='undef'
$d_sockatmark='undef'
$d_sockatmarkproto='undef'
$d_socket='define'
$d_sockpair='undef'
$d_socks5_init='undef'
$d_sqrtl='undef'
$d_srand48_r='undef'
$d_srandom_r='undef'
$d_sresgproto='undef'
$d_sresuproto='undef'
$d_statblks='undef'
$d_statfs_f_flags='undef'
$d_statfs_s='undef'
$d_stdio_cnt_lval='define'
$d_stdio_ptr_lval='define'
$d_stdio_ptr_lval_nochange_cnt='undef'
$d_stdio_ptr_lval_sets_cnt='undef'
$d_stdio_stream_array='define'
$d_stdiobase='define'
$d_stdstdio='define'
$d_strchr='define'
$d_strcoll='define'
$d_strctcpy='define'
$d_strerrm='strerror(e)'
$d_strerror='define'
$d_strerror_r='undef'
$d_strftime='define'
$d_strtod='define'
$d_strtol='define'
$d_strtold='undef'
$d_strtoll='undef'
$d_strtoq='undef'
$d_strtoul='define'
$d_strtoull='undef'
$d_strtouq='undef'
$d_strxfrm='define'
$d_suidsafe='define'
$d_symlink='define'
$d_syscall='undef'
$d_syscallproto='undef'
$d_sysconf='define'
$d_syserrlst='define'
$d_system='define'
$d_tcgetpgrp='undef'
$d_tcsetpgrp='undef'
$d_telldir='undef'
$d_telldirproto='undef'
$d_time='define'
$d_times='define'
$d_tm_tm_gmtoff='undef'
$d_tm_tm_zone='undef'
$d_tmpnam_r='undef'
$d_truncate='undef'
$d_ttyname_r='undef'
$d_tzname='define'
$d_u32align='define'
$d_ualarm='undef'
$d_umask='define'
$d_uname='define'
$d_union_semun='undef'
$d_unordered='undef'
$d_usleep='undef'
$d_usleepproto='undef'
$d_ustat='undef'
$d_vendorarch='define'
$d_vendorlib='define'
$d_vfork='undef'
$d_void_closedir='undef'
$d_volatile='define'
$d_vprintf='define'
$d_wait4='undef'
$d_waitpid='define'
$d_wcstombs='define'
$d_wctomb='define'
$d_writev='undef'
$db_hashtype='int'
$db_prefixtype='int'
$db_version_major='undef'
$db_version_minor='undef'
$db_version_patch='undef'
$defvoidused='15'
$direntrytype='struct dirent'
$doublesize='8'
$drand01='(rand() / (double) ((unsigned long)1 << 15))'
$drand48_r_proto='0'
$eagain='EAGAIN'
$ebcdic='undef'
$endgrent_r_proto='0'
$endhostent_r_proto='0'
$endnetent_r_proto='0'
$endprotoent_r_proto='0'
$endpwent_r_proto='0'
$endservent_r_proto='0'
$fflushNULL='define'
$fflushall='undef'
$fpostype='fpos_t'
$freetype='void'
$full_csh=''
$full_sed='/system/ported/command_library/sed.pm'
$getgrent_r_proto='0'
$getgrgid_r_proto='0'
$getgrnam_r_proto='0'
$gethostbyaddr_r_proto='0'
$gethostbyname_r_proto='0'
$gethostent_r_proto='0'
$getlogin_r_proto='0'
$getnetbyaddr_r_proto='0'
$getnetbyname_r_proto='0'
$getnetent_r_proto='0'
$getprotobyname_r_proto='0'
$getprotobynumber_r_proto='0'
$getprotoent_r_proto='0'
$getpwent_r_proto='0'
$getpwnam_r_proto='0'
$getpwuid_r_proto='0'
$getservbyname_r_proto='0'
$getservbyport_r_proto='0'
$getservent_r_proto='0'
$getspnam_r_proto='0'
$gidformat='"d"'
$gidsign='-1'
$gidsize='4'
$gidtype='gid_t'
$gmtime_r_proto='0'
$groupstype='gid_t'
$i16size='2'
$i16type='short'
$i32size='4'
$i32type='int'
$i64size='_error_'
$i64type='_error_'
$i8size='1'
$i8type='char'
$i_arpainet='define'
$i_crypt='undef'
$i_dbm='undef'
$i_dirent='define'
$i_dlfcn='undef'
$i_fcntl='define'
$i_float='define'
$i_fp='undef'
$i_fp_class='undef'
$i_grp='define'
$i_ieeefp='undef'
$i_inttypes='undef'
$i_langinfo='undef'
$i_libutil='undef'
$i_limits='define'
$i_locale='define'
$i_machcthr='undef'
$i_math='define'
$i_memory='undef'
$i_mntent='undef'
$i_ndbm='undef'
$i_netdb='define'
$i_neterrno='undef'
$i_netinettcp='undef'
$i_niin='define'
$i_poll='undef'
$i_prot='undef'
$i_pthread='undef'
$i_pwd='define'
$i_rpcsvcdbm='undef'
$i_sfio='undef'
$i_sgtty='undef'
$i_shadow='undef'
$i_socks='undef'
$i_stdarg='define'
$i_stddef='define'
$i_stdlib='define'
$i_string='define'
$i_sunmath='undef'
$i_sysaccess='undef'
$i_sysdir='undef'
$i_sysfile='undef'
$i_sysioctl='define'
$i_syslog='undef'
$i_sysmode='undef'
$i_sysmount='undef'
$i_sysndir='undef'
$i_sysparam='undef'
$i_sysresrc='undef'
$i_syssecrt='undef'
$i_sysselct='define'
$i_syssockio='undef'
$i_sysstat='define'
$i_sysstatfs='undef'
$i_sysstatvfs='undef'
$i_systime='define'
$i_systimek='undef'
$i_systimes='define'
$i_systypes='define'
$i_sysuio='undef'
$i_sysun='undef'
$i_sysutsname='define'
$i_sysvfs='undef'
$i_syswait='define'
$i_termio='undef'
$i_termios='define'
$i_time='undef'
$i_unistd='define'
$i_ustat='undef'
$i_utime='define'
$i_values='define'
$i_varargs='undef'
$i_vfork='undef'
$inc_version_list_init='0'
$installprefix='/system/ported/perl5'
$installprefixexp='/system/ported/perl5'
$installusrbinperl='undef'
$intsize='4'
$ivdformat='"d"'
$ivsize='4'
$ivtype='int'
$localtime_r_proto='0'
$longdblsize='8'
$longlongsize='_error_'
$longsize='4'
$lseeksize='4'
$lseektype='off_t'
$malloctype='void *'
$mmaptype='void *'
$modetype='mode_t'
$multiarch='undef'
$myuname='VOS'
$need_va_copy='undef'
$netdb_hlen_type='int'
$netdb_host_type='char *'
$netdb_name_type='char *'
$netdb_net_type='long'
$nveformat='"e"'
$nvfformat='"f"'
$nvgformat='"g"'
$nvsize='8'
$nvtype='double'
$o_nonblock='O_NONBLOCK'
$old_pthread_create_joinable=''
$osname='VOS'
$osvers='VOS'
$otherlibdirs=''
$package='perl5'
$pidtype='pid_t'
$pm_apiversion='5.005'
$privlib='/system/ported/lib/perl5/5.8.8'
$privlibexp='/system/ported/lib/perl5/5.8.8'
$procselfexe=''
$prototype='define'
$ptrsize='4'
$quadkind='_error_'
$quadtype='_error_'
$randbits='15'
$random_r_proto='0'
$randseedtype='unsigned int'
$rd_nodata='-1'
$readdir64_r_proto='0'
$readdir_r_proto='0'
$sPRIeldbl='"Le"'
$sPRIfldbl='"Lf"'
$sPRIgldbl='"Lg"'
$sSCNfldbl='"Lf"'
$sched_yield=''
$seedfunc='srand'
$selectminbits='1'
$selecttype='fd_set *'
$setgrent_r_proto='0'
$sethostent_r_proto='0'
$setlocale_r_proto='0'
$setnetent_r_proto='0'
$setprotoent_r_proto='0'
$setpwent_r_proto='0'
$setservent_r_proto='0'
$sh='/bin/sh'
$shmattype='void *'
$shortsize='2'
$sig_name_init='"ZERO","ABRT","FPE","ILL","INT","SEGV","TERM","USR1","USR2","IO","HUP","URG","ALRM","KILL","PIPE","QUIT","CHLD","CONT","STOP","TSTP","TTIN","TTOU","BUS","RT1","RT2","RT3","RT4","RT5","RT6","RT7","RT8",0'
$sig_num_init='0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,0'
$sig_size='31'
$signal_t='void'
$sitearch='/system/ported/lib/perl5/site_perl/5.8.8/hppa1.1'
$sitearchexp='/system/ported/lib/perl5/site_perl/5.8.8/hppa1.1'
$sitelib='/system/ported/lib/perl5/site_perl/5.8.8'
$sitelib_stem='/system/ported/lib/perl5/site_perl'
$sitelibexp='/system/ported/lib/perl5/site_perl/5.8.8'
$sizesize='4'
$sizetype='size_t'
$socksizetype='int'
$srand48_r_proto='0'
$srandom_r_proto='0'
$src='/vos_ftp_site/pub/vos/posix/(alpha|ga)/perl'
$ssizetype='ssize_t'
$startperl='!perl.pm'
$stdchar='unsigned char'
$stdio_base='((fp)->_base)'
$stdio_bufsiz='((fp)->_cnt + (fp)->_ptr - (fp)->_base)'
$stdio_cnt='((fp)->_cnt)'
$stdio_ptr='((fp)->_ptr)'
$stdio_stream_array='_iob'
$strerror_r_proto='0'
$targetarch='undef'
$timetype='time_t'
$tmpnam_r_proto='0'
$ttyname_r_proto='0'
$u16size='2'
$u16type='unsigned short'
$u32size='4'
$u32type='unsigned int'
$u64size='_error_'
$u64type='_error_'
$u8size='1'
$u8type='unsigned char'
$uidformat='"d"'
$uidsign='-1'
$uidsize='4'
$uidtype='uid_t'
$undef='$undef'
$uquadtype='_error_'
$use5005threads='undef'
$use64bitall='undef'
$use64bitint='undef'
$usecrosscompile='undef'
$usedl='undef'
$usefaststdio='define'
$usefaststdio='define'
$useithreads='undef'
$uselargefiles='undef'
$uselongdouble='define'
$usemorebits='undef'
$usemultiplicity='undef'
$useperlio='undef'
$usereentrant='undef'
$usesocks='undef'
$uvXUformat='"X"'
$uvoformat='"o"'
$uvsize='4'
$uvtype='unsigned int'
$uvuformat='"u"'
$uvxformat='"x"'
$vendorarch=''
$vendorarchexp=''
$vendorlib_stem=''
$vendorlibexp=''
$voidflags='15'
$xs_apiversion='5.00563'
$d_futimes='undef'
$d_malloc_size='undef'
$d_malloc_good_size='undef'
$usesitecustomize='undef'
$d_unsetenv='undef'
$usemallocwrap='undef'
$d_nv_zero_is_allbits_zero='undef'
$d_attribute_format='undef'
$d_attribute_noreturn='undef'
$d_attribute_malloc='undef'
$d_attribute_nonnull='undef'
$d_attribute_pure='undef'
$d_attribute_unused='undef'
$d_attribute_warn_unused_result='undef'
$d_clearenv='undef'
$d_libm_lib_version='undef'
$d_sprintf_returns_strlen='undef'
$d_strlcat='undef'
$d_strlcpy='undef'
$userelocatableinc='undef'
